"""CRITIC structured output module."""

from typing import Any, Dict

from pydantic import BaseModel, Field


class CriticOutput(BaseModel):
    """Critic Pydantic output class.

    Attributes:
        answer (str): The answer generated by the agent.
        critique (str): The critique of the answer generated by the agent.
        external_tool_info (Dict[str, Any]): The query requested by the agent.
    """

    answer: str = Field(..., description="The answer generated by the agent.")
    critique: str = Field(..., description="The answer's critique.")
    external_tool_info: Dict[str, Any] = Field(
        ..., description="The external tool outputs."
    )
